@@ -9,10 +9,12 @@
 #ifndef GT_ALIGNMENT_H_
 #define GT_ALIGNMENT_H_
 
-#include "gt_commons.h"
+#include "gt_essentials.h"
+#include "gt_attributes.h"
+
 #include "gt_map.h"
+
 #include "gt_input_parser.h"
-#include "gt_data_attributes.h"
 
 // Alignment itself
 typedef struct _gt_alignment_dictionary gt_alignment_dictionary; // Forward declaration of gt_alignment_dictionary
@@ -27,8 +29,8 @@ typedef struct {
   gt_vector* counters;
   /* Maps structures */
   gt_vector* maps; /* (gt_map*) */
-  char* maps_txt;
-  gt_shash* attributes;
+  /* Attibutes */
+  gt_attributes* attributes;
   /* Hashed Dictionary */
   gt_alignment_dictionary* alg_dictionary;
 } gt_alignment;
@@ -44,8 +46,13 @@ typedef struct {
   gt_ihash* begin_position; /* (uint64_t) */
   gt_ihash* end_position;   /* (uint64_t) */
 } gt_alignment_dictionary_element;
+typedef struct {
+  gt_map* map; /* the map*/
+  uint64_t pos; /* the map position in the alignment map list */
+} gt_alignment_dictionary_map_element;
 struct _gt_alignment_dictionary {
   gt_shash* maps_dictionary; /* (gt_alignment_dictionary_element*) */
+  gt_shash* refs_dictionary; /* (gt_alignment_dictionary_map_element*) */
   gt_alignment* alignment;
 };
 
@@ -59,10 +66,11 @@ struct _gt_alignment_dictionary {
   GT_STRING_CHECK(alignment->qualities); \
   GT_VECTOR_CHECK(alignment->counters); \
   GT_VECTOR_CHECK(alignment->maps); \
-  GT_HASH_CHECK(alignment->attributes)
+  GT_ATTRIBUTES_CHECK(alignment->attributes)
 #define GT_ALIGNMENT_DICTIONARY_CHECK(alignment_dictionary) \
   GT_NULL_CHECK(alignment_dictionary); \
-  GT_HASH_CHECK(alignment_dictionary->maps_dictionary)
+  GT_HASH_CHECK(alignment_dictionary->maps_dictionary); \
+  GT_HASH_CHECK(alignment_dictionary->refs_dictionary)
 
 /*
  *  TODO: Scheduled for v2.0 (all lazy parsing)
@@ -106,15 +114,13 @@ GT_INLINE void gt_alignment_inc_counter(gt_alignment* const alignment,const uint
 /*
  * Attribute accessors
  */
-// Predefined attributes
-#define GT_ATTR_MAX_COMPLETE_STRATA "MCS"
-#define GT_ATTR_NOT_UNIQUE "NOT-UNIQUE"
-
 GT_INLINE uint64_t gt_alignment_get_mcs(gt_alignment* const alignment);
 GT_INLINE void gt_alignment_set_mcs(gt_alignment* const alignment,uint64_t max_complete_strata);
 GT_INLINE void gt_alignment_set_not_unique_flag(gt_alignment* const alignment,bool is_not_unique);
 GT_INLINE bool gt_alignment_get_not_unique_flag(gt_alignment* const alignment);
 GT_INLINE int64_t gt_alignment_get_pair(gt_alignment* const alignment);
+GT_INLINE void gt_alignment_set_map_primary(gt_alignment* const alignment,gt_map* const map);
+GT_INLINE gt_map* gt_alignment_get_map_primary(gt_alignment* const alignment);
 
 /*
  * Maps Handlers
@@ -146,7 +152,7 @@ GT_INLINE uint64_t gt_alignment_next_map_pos(gt_alignment_map_iterator* const al
 #define GT_ALIGNMENT_ITERATE(alignment,map) \
   /* Alignment. Iterate over all maps */ \
   gt_alignment_map_iterator __##map##_iterator; \
-  register gt_map* map; \
+  gt_map* map; \
   gt_alignment_new_map_iterator(alignment,&(__##map##_iterator)); \
   while ((map=gt_alignment_next_map(&(__##map##_iterator))))
 
@@ -155,6 +161,7 @@ GT_INLINE uint64_t gt_alignment_next_map_pos(gt_alignment_map_iterator* const al
  */
 GT_INLINE gt_alignment_dictionary* gt_alignment_dictionary_new(gt_alignment* const alignment);
 GT_INLINE void gt_alignment_dictionary_delete(gt_alignment_dictionary* const alignment_dictionary);
+GT_INLINE void gt_alignment_dictionary_add_ref(gt_alignment_dictionary* const alignment_dictionary, gt_alignment* const alignment);
 GT_INLINE bool gt_alignment_dictionary_try_add(
     gt_alignment_dictionary* const alignment_dictionary,gt_map* const map,
     const uint64_t begin_position,const uint64_t end_position,
