@@ -9,7 +9,7 @@
 #ifndef GT_SHASH_H_
 #define GT_SHASH_H_
 
-#include "gt_commons.h"
+#include "gt_hash.h"
 #include "uthash.h"
 
 /*
@@ -28,6 +28,10 @@ typedef struct {
 typedef struct {
   gt_shash_element* shash_head;
 } gt_shash;
+typedef struct {
+  gt_shash_element* shash;
+  gt_shash_element* next;
+} gt_shash_iterator;
 
 /*
  * Constructor
@@ -35,6 +39,7 @@ typedef struct {
 GT_INLINE gt_shash* gt_shash_new(void);
 GT_INLINE void gt_shash_clear(gt_shash* const shash,const bool free_element);
 GT_INLINE void gt_shash_delete(gt_shash* const shash,const bool free_element);
+GT_INLINE void gt_shash_destroy(gt_shash* const shash);
 
 /*
  * Basic (Type-unsafe) Accessors
@@ -70,19 +75,27 @@ GT_INLINE void gt_shash_copy(gt_shash* const shash_dst,gt_shash* const shash_src
 #define GT_SHASH_BEGIN_ITERATE(shash,it_skey,it_element,type) { \
   gt_shash_element *shash_##sh_element, *shash_##tmp; \
   HASH_ITER(hh,shash->shash_head,shash_##sh_element,shash_##tmp) { \
-    register type* const it_element = (type*)(shash_##sh_element->element); \
-    register char* const it_skey = shash_##sh_element->key;
+    type* const it_element = (type*)(shash_##sh_element->element); \
+    char* const it_skey = shash_##sh_element->key;
 
 #define GT_SHASH_BEGIN_ELEMENT_ITERATE(shash,it_element,type) { \
   gt_shash_element *shash_##sh_element, *shash_##tmp; \
   HASH_ITER(hh,shash->shash_head,shash_##sh_element,shash_##tmp) { \
-    register type* const it_element = (type*)(shash_##sh_element->element);
+    type* const it_element = (type*)(shash_##sh_element->element);
 
-#define GT_SHASH_BEGIN_KEY_ITERATE(shash,it_skey,type) { \
+#define GT_SHASH_BEGIN_KEY_ITERATE(shash,it_skey) { \
   gt_shash_element *shash_##sh_element, *shash_##tmp; \
   HASH_ITER(hh,shash->shash_head,shash_##sh_element,shash_##tmp) { \
-    register char* const it_skey = shash_##sh_element->key;
+    char* const it_skey = shash_##sh_element->key;
 
 #define GT_SHASH_END_ITERATE }}
 
+GT_INLINE gt_shash_iterator* gt_shash_iterator_new(gt_shash* const shash);
+GT_INLINE void gt_shash_iterator_delete(gt_shash_iterator* const shash_iterator);
+
+GT_INLINE bool gt_shash_iterator_next(gt_shash_iterator* const shash_iterator);
+GT_INLINE char* gt_shash_iterator_get_key(gt_shash_iterator* const shash_iterator);
+GT_INLINE void* gt_shash_iterator_get_element(gt_shash_iterator* const shash_iterator);
+
+
 #endif /* GT_SHASH_H_ */
