@@ -7,48 +7,15 @@
  */
 
 #include "gt_commons.h"
+#include "gt_error.h"
 
 /*
- * String/Buffer functions
+ * Helper functions (OPERATIVE)
  */
-GT_INLINE void gt_strncpy(char* const buffer_dst,char* const buffer_src,const uint64_t length) {
-  GT_NULL_CHECK(buffer_dst); GT_NULL_CHECK(buffer_src);
-  memcpy(buffer_dst,buffer_src,length);
-  buffer_dst[length] = EOS;
-}
-GT_INLINE char* gt_strndup(char* const buffer,const uint64_t length) {
-  GT_NULL_CHECK(buffer);
-  register char* const buffer_cpy = malloc(length+1);
-  gt_cond_fatal_error(!buffer_cpy,MEM_ALLOC);
-  gt_strncpy(buffer_cpy,buffer,length);
-  return buffer_cpy;
-}
-GT_INLINE int gt_strcmp(char* const buffer_a,char* const buffer_b) {
-  GT_NULL_CHECK(buffer_a); GT_NULL_CHECK(buffer_b);
-  return strcmp(buffer_a,buffer_b);
-}
-GT_INLINE bool gt_streq(char* const buffer_a,char* const buffer_b) { // TODO: Add safe mem str_cmp
-  GT_NULL_CHECK(buffer_a); GT_NULL_CHECK(buffer_b);
-  return strcmp(buffer_a,buffer_b)==0;
-}
-GT_INLINE int gt_strncmp(char* const buffer_a,char* const buffer_b,const uint64_t length) {
-  GT_NULL_CHECK(buffer_a); GT_NULL_CHECK(buffer_b);
-  return strncmp(buffer_a,buffer_b,length);
-}
-GT_INLINE bool gt_strneq(char* const buffer_a,char* const buffer_b,const uint64_t length) {
-  GT_NULL_CHECK(buffer_a); GT_NULL_CHECK(buffer_b);
-  return strncmp(buffer_a,buffer_b,length)==0;
-}
-
-/*
- * Memory usage helper functions
- */
-GT_INLINE void* gt_malloc_(uint64_t const num_elements,const uint64_t size_element,const bool init_mem,const int init_value) {
-  register const uint64_t total_memory = num_elements*size_element;
-  register void* const allocated_mem = malloc(total_memory);
-  gt_cond_fatal_error(!allocated_mem,MEM_HANDLER);
-  if (gt_expect_false(init_mem)) memset(allocated_mem,init_value,total_memory);
-  return allocated_mem;
+GT_INLINE uint64_t gt_get_integer_proportion(const float proportion,const uint64_t length) {
+  if (proportion<=0.0) return 0;
+  if (proportion>=1.0) return (uint64_t)proportion;
+  return (uint64_t)(proportion*(float)length);
 }
 
 /*
@@ -60,8 +27,8 @@ GT_INLINE uint64_t gt_calculate_memory_required_v(const char *template,va_list v
   va_list v_args_cpy;
   va_copy(v_args_cpy,v_args);
   // Calculate memory required to print the template{v_args}
-  register uint64_t mem_required = 0, precision=0;
-  register const char* centinel;
+  uint64_t mem_required = 0, precision=0;
+  const char* centinel;
   for (centinel=template;*centinel!=EOS;++centinel,precision=0) {
     if (*centinel==FORMAT) {
       ++centinel;
@@ -80,7 +47,7 @@ GT_INLINE uint64_t gt_calculate_memory_required_v(const char *template,va_list v
       // Check format
       switch (*centinel) {
         case 's': { // String requires fetching the argument length // FIXME: %.*s
-          register char* const string = va_arg(v_args_cpy,char*);
+          char* const string = va_arg(v_args_cpy,char*);
           mem_required += (precision>0) ? precision : strlen(string);
           break;
         }
@@ -103,3 +70,19 @@ GT_INLINE uint64_t gt_calculate_memory_required_va(const char *template,...) {
   va_start(v_args,template);
   return gt_calculate_memory_required_v(template,v_args);
 }
+
+/*
+ * Random number generator
+ */
+GT_INLINE uint64_t gt_rand_IID(const uint64_t min,const uint64_t max) {
+  int n_rand = rand(); // [0, RAND_MAX]
+  const uint64_t range = max - min;
+  const uint64_t rem = RAND_MAX % range;
+  const uint64_t sample = RAND_MAX / range;
+  // Consider the small interval within remainder of RAND_MAX
+  if (n_rand < RAND_MAX - rem) {
+    return min + n_rand/sample;
+  } else {
+    return gt_rand_IID(min,max);
+  }
+}
