@@ -8,31 +8,9 @@
 
 #include "gt_compact_dna_string.h"
 
-/*
- * Blocks encoding dimensions
- */
-#define GT_CDNA_BLOCK_CHARS 64
-#define GT_CDNA_BLOCK_BITMAP_SIZE 8
-#define GT_CDNA_BLOCK_BITMAPS 3
-#define GT_CDNA_BLOCK_SIZE (GT_CDNA_BLOCK_BITMAP_SIZE*GT_CDNA_BLOCK_BITMAPS)
-
 /*
  * CDNA Bitmaps Encoding
  */
-#define GT_CDNA_ENC_CHAR_A 0
-#define GT_CDNA_ENC_CHAR_C 1
-#define GT_CDNA_ENC_CHAR_G 2
-#define GT_CDNA_ENC_CHAR_T 3
-#define GT_CDNA_ENC_CHAR_N 4
-
-#define GT_CDNA_ZERO_MASK       0xFFFFFFFFFFFFFFFEull
-#define GT_CDNA_ONE_MASK        0x0000000000000001ull
-#define GT_CDNA_ONE_LAST_MASK   0x8000000000000000ull
-#define GT_CDNA_ZERO_LAST_MASK  0x7FFFFFFFFFFFFFFFull
-
-#define GT_CDNA_EXTRACT_MASK      GT_CDNA_ONE_MASK
-#define GT_CDNA_EXTRACT_LAST_MASK GT_CDNA_ONE_LAST_MASK
-
 #define GT_CDNA_ENCODED_CHAR_BM0 ((uint8_t)1)
 #define GT_CDNA_ENCODED_CHAR_BM1 ((uint8_t)2)
 #define GT_CDNA_ENCODED_CHAR_BM2 ((uint8_t)4)
@@ -85,7 +63,7 @@ const uint8_t gt_cdna_encode[256] = {
 }
 
 #define GT_CDNA_GET_BLOCKS(bitmaps_mem,block_num,bm_0,bm_1,bm_2) { \
-  register uint64_t* const block_mem = GT_CDNA_GET_MEM_BLOCK(bitmaps_mem,block_num); \
+  uint64_t* const block_mem = GT_CDNA_GET_MEM_BLOCK(bitmaps_mem,block_num); \
   GT_CDNA_LOAD_BLOCKS(block_mem,bm_0,bm_1,bm_2); \
 }
 
@@ -113,8 +91,8 @@ const uint8_t gt_cdna_encode[256] = {
   }
 
 #define GT_CDNA_SET_CHAR(block_mem,block_pos,enc_char)  \
-  register const uint64_t bm_one_mask = GT_CDNA_ONE_MASK<<(block_pos); \
-  register const uint64_t bm_zero_mask = ~(bm_one_mask); \
+  const uint64_t bm_one_mask = GT_CDNA_ONE_MASK<<(block_pos); \
+  const uint64_t bm_zero_mask = ~(bm_one_mask); \
   GT_CDNA_PROYECT_CHAR(block_mem,enc_char,0,bm_one_mask,bm_zero_mask); \
   GT_CDNA_PROYECT_CHAR(block_mem,enc_char,1,bm_one_mask,bm_zero_mask); \
   GT_CDNA_PROYECT_CHAR(block_mem,enc_char,2,bm_one_mask,bm_zero_mask)
@@ -123,10 +101,9 @@ const uint8_t gt_cdna_encode[256] = {
  * Constructor
  */
 GT_INLINE gt_compact_dna_string* gt_cdna_string_new(const uint64_t initial_chars) {
-  gt_compact_dna_string* cdna_string = malloc(sizeof(gt_compact_dna_string));
-  gt_cond_fatal_error(!cdna_string,MEM_HANDLER);
-  register const uint64_t initial_blocks = initial_chars ? GT_CDNA_GET_NUM_BLOCKS(initial_chars) : 1;
-  cdna_string->bitmaps = malloc(GT_CDNA_GET_BLOCKS_MEM(initial_blocks));
+  gt_compact_dna_string* cdna_string = gt_alloc(gt_compact_dna_string);
+  const uint64_t initial_blocks = initial_chars ? GT_CDNA_GET_NUM_BLOCKS(initial_chars) : 1;
+  cdna_string->bitmaps = gt_malloc(GT_CDNA_GET_BLOCKS_MEM(initial_blocks));
   gt_cond_fatal_error(!cdna_string->bitmaps,MEM_ALLOC);
   cdna_string->allocated = GT_CDNA_GET_NUM_CHARS(initial_blocks);
   cdna_string->length = 0;
@@ -136,7 +113,7 @@ GT_INLINE gt_compact_dna_string* gt_cdna_string_new(const uint64_t initial_chars
 GT_INLINE void gt_cdna_string_resize(gt_compact_dna_string* const cdna_string,const uint64_t num_chars) {
   GT_COMPACT_DNA_STRING_CHECK(cdna_string);
   if (num_chars > cdna_string->allocated) {
-    register const uint64_t num_blocks = GT_CDNA_GET_NUM_BLOCKS(num_chars);
+    const uint64_t num_blocks = GT_CDNA_GET_NUM_BLOCKS(num_chars);
     cdna_string->bitmaps=realloc(cdna_string->bitmaps,GT_CDNA_GET_BLOCKS_MEM(num_blocks));
     gt_cond_fatal_error(!cdna_string->bitmaps,MEM_REALLOC);
     cdna_string->allocated = GT_CDNA_GET_NUM_CHARS(num_blocks);
@@ -149,8 +126,8 @@ GT_INLINE void gt_cdna_string_clear(gt_compact_dna_string* const cdna_string) {
 }
 GT_INLINE void gt_cdna_string_delete(gt_compact_dna_string* const cdna_string) {
   GT_COMPACT_DNA_STRING_CHECK(cdna_string);
-  free(cdna_string->bitmaps);
-  free(cdna_string);
+  gt_free(cdna_string->bitmaps);
+  gt_free(cdna_string);
 }
 
 /*
@@ -162,11 +139,11 @@ GT_INLINE void gt_cdna_allocate__init_blocks(gt_compact_dna_string* const cdna_s
     // Check allocated blocks
     if (pos >= cdna_string->allocated) gt_cdna_string_resize(cdna_string,pos+(GT_CDNA_BLOCK_CHARS*10));
     // Initialize new accessed blocks
-    register const uint64_t next_block_num = gt_expect_true(cdna_string->length>0) ? ((cdna_string->length-1)/GT_CDNA_BLOCK_CHARS)+1 : 1;
-    register const uint64_t top_block_num = pos/GT_CDNA_BLOCK_CHARS;
+    const uint64_t next_block_num = gt_expect_true(cdna_string->length>0) ? ((cdna_string->length-1)/GT_CDNA_BLOCK_CHARS)+1 : 1;
+    const uint64_t top_block_num = pos/GT_CDNA_BLOCK_CHARS;
     if (next_block_num <= top_block_num) {
-      register uint64_t i;
-      register uint64_t* block_mem = GT_CDNA_GET_MEM_BLOCK(cdna_string->bitmaps,next_block_num);
+      uint64_t i;
+      uint64_t* block_mem = GT_CDNA_GET_MEM_BLOCK(cdna_string->bitmaps,next_block_num);
       for (i=next_block_num; i<=top_block_num; ++i) {
         GT_CDNA_INIT_BLOCK(block_mem);
         block_mem+=GT_CDNA_BLOCK_BITMAPS;
@@ -179,8 +156,8 @@ GT_INLINE void gt_cdna_allocate__init_blocks(gt_compact_dna_string* const cdna_s
 GT_INLINE char gt_cdna_string_get_char_at(gt_compact_dna_string* const cdna_string,const uint64_t position) {
   GT_COMPACT_DNA_STRING_CHECK(cdna_string);
   GT_COMPACT_DNA_STRING_POSITION_CHECK(cdna_string,position);
-  register uint64_t block_num, block_pos;
-  register uint64_t bm_0, bm_1, bm_2;
+  uint64_t block_num, block_pos;
+  uint64_t bm_0, bm_1, bm_2;
   GT_CDNA_GET_BLOCK_POS(position,block_num,block_pos);
   GT_CDNA_GET_BLOCKS(cdna_string->bitmaps,block_num,bm_0,bm_1,bm_2);
   GT_CDNA_SHIFT_FORWARD_CHARS(block_pos,bm_0,bm_1,bm_2);
@@ -191,10 +168,10 @@ GT_INLINE void gt_cdna_string_set_char_at(gt_compact_dna_string* const cdna_stri
   // Check allocated bitmaps
   gt_cdna_allocate__init_blocks(cdna_string,position);
   // Encode char
-  register uint64_t block_num, block_pos;
+  uint64_t block_num, block_pos;
   GT_CDNA_GET_BLOCK_POS(position,block_num,block_pos);
-  register uint64_t* const block_mem = GT_CDNA_GET_MEM_BLOCK(cdna_string->bitmaps,block_num);
-  register const uint8_t enc_char = gt_cdna_encode(character);
+  uint64_t* const block_mem = GT_CDNA_GET_MEM_BLOCK(cdna_string->bitmaps,block_num);
+  const uint8_t enc_char = gt_cdna_encode(character);
   GT_CDNA_SET_CHAR(block_mem,block_pos,enc_char);
 }
 GT_INLINE uint64_t gt_cdna_string_get_length(gt_compact_dna_string* const cdna_string) {
@@ -202,23 +179,23 @@ GT_INLINE uint64_t gt_cdna_string_get_length(gt_compact_dna_string* const cdna_s
   return cdna_string->length;
 }
 
-GT_INLINE void gt_cdna_string_append_string(gt_compact_dna_string* const cdna_string,char* const string,const uint64_t length) {
+GT_INLINE void gt_cdna_string_append_string(gt_compact_dna_string* const cdna_string,const char* const string,const uint64_t length) {
   GT_COMPACT_DNA_STRING_CHECK(cdna_string);
   // Check allocated bitmaps
-  register const uint64_t total_chars = cdna_string->length+length-1;
+  const uint64_t total_chars = cdna_string->length+length-1;
   if (total_chars >= cdna_string->allocated) {
     gt_cdna_string_resize(cdna_string,total_chars);
   }
   // Copy string
-  register uint64_t block_num, block_pos, i;
+  uint64_t block_num, block_pos, i;
   GT_CDNA_GET_BLOCK_POS(cdna_string->length,block_num,block_pos);
-  register uint64_t* block_mem = GT_CDNA_GET_MEM_BLOCK(cdna_string->bitmaps,block_num);
+  uint64_t* block_mem = GT_CDNA_GET_MEM_BLOCK(cdna_string->bitmaps,block_num);
   for (i=0; i<length; ++i,++block_pos) {
     if (gt_expect_false(block_pos==GT_CDNA_BLOCK_CHARS)) {
       block_pos=0;
       block_mem+=GT_CDNA_BLOCK_BITMAPS;
     }
-    register const uint8_t enc_char = gt_cdna_encode(string[i]);
+    const uint8_t enc_char = gt_cdna_encode(string[i]);
     GT_CDNA_SET_CHAR(block_mem,block_pos,enc_char);
   }
   // Update total length
@@ -230,7 +207,7 @@ GT_INLINE void gt_cdna_string_append_string(gt_compact_dna_string* const cdna_st
  */
 GT_INLINE void gt_cdna_string_new_iterator(
     gt_compact_dna_string* const cdna_string,const uint64_t position,gt_string_traversal const direction,
-    gt_compact_dna_sequence_iterator* const cdna_string_iterator) {
+    gt_compact_dna_string_iterator* const cdna_string_iterator) {
   GT_COMPACT_DNA_STRING_CHECK(cdna_string);
   GT_NULL_CHECK(cdna_string_iterator);
   // Initialize the iterator
@@ -243,18 +220,18 @@ GT_INLINE void gt_cdna_string_new_iterator(
   }
 }
 GT_INLINE void gt_cdna_string_iterator_seek(
-    gt_compact_dna_sequence_iterator* const cdna_string_iterator,
+    gt_compact_dna_string_iterator* const cdna_string_iterator,
     const uint64_t position,gt_string_traversal const direction) {
   GT_NULL_CHECK(cdna_string_iterator);
   GT_NULL_CHECK(cdna_string_iterator->cdna_string);
-  register gt_compact_dna_string* const cdna_string = cdna_string_iterator->cdna_string;
+  gt_compact_dna_string* const cdna_string = cdna_string_iterator->cdna_string;
   GT_COMPACT_DNA_STRING_POSITION_CHECK(cdna_string,position);
   // Set the iterator
   cdna_string_iterator->current_pos = position;
   cdna_string_iterator->current_pos_mod = position%GT_CDNA_BLOCK_CHARS;
   cdna_string_iterator->direction = direction;
   // Set the current bitmap and seek to current position
-  register uint64_t block_num, block_pos;
+  uint64_t block_num, block_pos;
   GT_CDNA_GET_BLOCK_POS(position,block_num,block_pos);
   cdna_string_iterator->current_bitmap = GT_CDNA_GET_MEM_BLOCK(cdna_string->bitmaps,block_num);
   GT_CDNA_GET_BLOCKS(cdna_string->bitmaps,block_num,cdna_string_iterator->bm_0,cdna_string_iterator->bm_1,cdna_string_iterator->bm_2);
@@ -266,15 +243,15 @@ GT_INLINE void gt_cdna_string_iterator_seek(
         cdna_string_iterator->bm_0,cdna_string_iterator->bm_1,cdna_string_iterator->bm_2);
   }
 }
-GT_INLINE bool gt_cdna_string_iterator_eos(gt_compact_dna_sequence_iterator* const cdna_string_iterator) {
+GT_INLINE bool gt_cdna_string_iterator_eos(gt_compact_dna_string_iterator* const cdna_string_iterator) {
   GT_COMPACT_DNA_STRING_ITERATOR_CHECK(cdna_string_iterator);
   return cdna_string_iterator->current_pos>=cdna_string_iterator->cdna_string->length;
 }
-GT_INLINE char gt_cdna_string_iterator_following(gt_compact_dna_sequence_iterator* const cdna_string_iterator) {
+GT_INLINE char gt_cdna_string_iterator_following(gt_compact_dna_string_iterator* const cdna_string_iterator) {
   GT_COMPACT_DNA_STRING_ITERATOR_CHECK(cdna_string_iterator);
   GT_COMPACT_DNA_STRING_POSITION_CHECK(cdna_string_iterator->cdna_string,cdna_string_iterator->current_pos);
   // Extract character
-  register const char character = gt_cdna_decode[GT_CDNA_EXTRACT_CHAR(cdna_string_iterator->bm_0,cdna_string_iterator->bm_1,cdna_string_iterator->bm_2)];
+  const char character = gt_cdna_decode[GT_CDNA_EXTRACT_CHAR(cdna_string_iterator->bm_0,cdna_string_iterator->bm_1,cdna_string_iterator->bm_2)];
   // Update position
   ++cdna_string_iterator->current_pos;
   // Seek to proper position (load block if necessary)
@@ -289,11 +266,11 @@ GT_INLINE char gt_cdna_string_iterator_following(gt_compact_dna_sequence_iterato
   // Return the character decoded
   return character;
 }
-GT_INLINE char gt_cdna_string_iterator_previous(gt_compact_dna_sequence_iterator* const cdna_string_iterator) {
+GT_INLINE char gt_cdna_string_iterator_previous(gt_compact_dna_string_iterator* const cdna_string_iterator) {
   GT_COMPACT_DNA_STRING_ITERATOR_CHECK(cdna_string_iterator);
   GT_COMPACT_DNA_STRING_POSITION_CHECK(cdna_string_iterator->cdna_string,cdna_string_iterator->current_pos);
   // Extract character
-  register const char character = gt_cdna_decode[GT_CDNA_EXTRACT_LAST_CHAR(cdna_string_iterator->bm_0,cdna_string_iterator->bm_1,cdna_string_iterator->bm_2)];
+  const char character = gt_cdna_decode[GT_CDNA_EXTRACT_LAST_CHAR(cdna_string_iterator->bm_0,cdna_string_iterator->bm_1,cdna_string_iterator->bm_2)];
   // Update position
   --cdna_string_iterator->current_pos;
   // Seek to proper position (load block if necessary)
@@ -308,7 +285,7 @@ GT_INLINE char gt_cdna_string_iterator_previous(gt_compact_dna_sequence_iterator
   // Return the character decoded
   return character;
 }
-GT_INLINE char gt_cdna_string_iterator_next(gt_compact_dna_sequence_iterator* const cdna_string_iterator) {
+GT_INLINE char gt_cdna_string_iterator_next(gt_compact_dna_string_iterator* const cdna_string_iterator) {
   GT_COMPACT_DNA_STRING_ITERATOR_CHECK(cdna_string_iterator);
   return (cdna_string_iterator->direction==GT_ST_FORWARD) ?
       gt_cdna_string_iterator_following(cdna_string_iterator) :
