@@ -17,8 +17,6 @@ import gem
 import gem.junctions
 import __builtin__
 
-import multiprocessing as mp
-
 
 LOG_NOTHING = 1
 LOG_STDERR = 2
@@ -128,15 +126,17 @@ class execs_dict(dict):
 executables = execs_dict({
     "gem-indexer": "gem-indexer",
     "gem-mapper": "gem-mapper",
-    "gem-rna-mapper": "gem-rna-mapper",
     "gem-2-gem": "gem-2-gem",
     "gem-2-sam": "gem-2-sam",
     "samtools": "samtools",
     "gem-info": "gem-info",
-    "splits-2-junctions": "splits-2-junctions",
     "gem-retriever": "gem-retriever",
-    "compute-transcriptome": "compute-transcriptome",
-    "transcriptome-2-genome": "transcriptome-2-genome",
+    "gem-rna-tools": "gem-rna-tools",
+    "gt.filter": "gt.filter",
+    "gt.map.2.sam": "gt.map.2.sam",
+    "gt.mapset": "gt.mapset",
+    "gt.gtfcount": "gt.gtfcount",
+    "gt.stats": "gt.stats"
     })
 
 
@@ -161,6 +161,14 @@ def loglevel(level):
     logging.gemtools.level = numeric_level
 
 
+# cleanup functions
+def _cleanup_on_shutdown():
+    gem.utils.terminate_processes()
+    gem.files._cleanup()
+
+import atexit
+atexit.register(_cleanup_on_shutdown)
+
 def _prepare_index_parameter(index, gem_suffix=True):
     """Prepares the index file and checks that the index
     exists. The function throws a IOError if the index file
@@ -255,23 +263,34 @@ def _prepare_output(process, output=None, quality=None, bam=False):
     output  -- output file name or None when process stdout should be used
     quality -- optional quality that is passed to the InputFile
     """
+    # wrap process list
+    if not isinstance(process, (list, tuple)):
+        process = [process]
+
     if output is not None:
         # we are writing to a file
         # wait for the process to finish
-        if process is not None and process.wait() != 0:
-            #raise ValueError("Execution failed!")
-            raise gem.utils.ProcessError("Execution failed!")
+        if process is not None:
+            for k, p in enumerate(process):
+                if p is not None and p.wait() != 0:
+                    raise gem.utils.ProcessError("Execution failed!")
+                # close streams next process in
+                if (k + 1) < len(process):
+                    next_process = process[k + 1]
+                    if next_process is not None and \
+                       next_process.stdin is not None:
+                        next_process.stdin.close()
         logging.debug("Opening output file %s" % (output))
         if output.endswith(".bam") or bam:
-            return gt.InputFile(gem.files.open_bam(output), quality=quality, process=process)
-        return gt.InputFile(output, quality=quality, process=process)
+            return gt.InputFile(gem.files.open_bam(output), quality=quality, process=process[0])
+        return gt.InputFile(output, quality=quality, process=process[0])
     else:
         logging.debug("Opening output stream")
         if bam:
-            return gt.InputFile(gem.files.open_bam(process.stdout), quality=quality, process=process)
+            return gt.InputFile(gem.files.open_bam(process[0].stdout), quality=quality, process=process[0])
         ## running in async mode, return iterator on
         ## the output stream
-        return gt.InputFile(process.stdout, quality=quality, process=process)
+        return gt.InputFile(process[0].stdout, quality=quality, process=process[0])
 
 
 def validate_executables():
@@ -390,7 +409,11 @@ def mapper(input, index, output=None,
 
     # convert to genome coordinates if mapping to transcriptome
     if key_file is not None:
-        convert_to_genome = [executables['transcriptome-2-genome'], key_file, str(max(1, threads / 2))]
+        convert_to_genome = [executables['gem-rna-tools'],
+                             'transcriptome-2-genome',
+                             '-k', key_file,
+                             '--threads', str(max(1, threads / 2))
+                             ]
         tools.append(convert_to_genome)
 
     if compress:
@@ -485,9 +508,9 @@ def transcript_mapper(input, indices, key_files, output=None,
            )
         )
     if len(indices) > 1:
-        merged = merger(outputs[0], outputs[1:])
+        merged = merge(outputs[0], outputs[1:], paired=False, threads=threads,
+                       same_content=True, output=output)
         if(output is not None):
-            merged.merge(output, threads=threads, same_content=True)
             for f in output_files:
                 os.remove(f)
             return _prepare_output(None, output=output, quality=quality)
@@ -536,7 +559,8 @@ def splitmapper(input,
     quality = _prepare_quality_parameter(quality)
     splice_cons = _prepare_splice_consensus_parameter(splice_consensus)
 
-    pa = [executables['gem-rna-mapper'],
+    pa = [executables['gem-rna-tools'],
+          'split-mapper',
           '-I', index,
           '-q', quality,
           '-m', str(mismatches),
@@ -662,6 +686,7 @@ def pairalign(input, index, output=None,
               max_matches_per_extension=0,
               unique_pairing=False,
               map_both_ends=False,
+              filter_max_matches=0,
               threads=1,
               compress=False,
               extra=None):
@@ -700,6 +725,10 @@ def pairalign(input, index, output=None,
         pa.append("--map-both-ends")
 
     tools = [pa]
+    filter_pa = [executables["gt.filter"], "-t", str(threads), "-p"]
+    if filter_max_matches > 0:
+        filter_pa.extend(["--max-output-matches", str(filter_max_matches)])
+    tools.append(filter_pa)
     if compress:
         gzip = _compressor(threads=max(1, threads / 2))
         tools.append(gzip)
@@ -785,22 +814,24 @@ def score(input,
 
     if raw or isinstance(input, gt.InputFile):
         raw = True
+        if isinstance(input, gt.InputFile):
+            input.remove_scores = True
         #input = input.raw_stream()
 
     tools = [score_p]
 
     if compress:
-        gzip = _compressor(threads=max(1, threads / 2))
+        gzip = _compressor(threads=threads)
         tools.append(gzip)
 
-    process = utils.run_tools(tools, input=input, output=output, name="GEM-Score", write_map=True, raw=raw)
+    process = utils.run_tools(tools, input=input, output=output, name="GEM-Score", write_map=True, raw=False)
     return _prepare_output(process, output=output)
 
 
 def gem2sam(input, index=None, output=None,
     single_end=False, compact=False, threads=1,
     quality=None, check_ids=True, add_length=True, consensus=None,
-    exclude_header=False):
+    exclude_header=False, calc_xs=True, raw=False):
 
     if index is not None:
         index = _prepare_index_parameter(index, gem_suffix=True)
@@ -814,14 +845,12 @@ def gem2sam(input, index=None, output=None,
         gem_2_sam_p.extend(['-I', index])
         if not exclude_header:
             gem_2_sam_p.append("-l")
-    if add_length is not None:
-        gem_2_sam_p.append('-l')
 
     quality = _prepare_quality_parameter(quality, input)
     if quality is not None and not quality == "ignore":
         gem_2_sam_p.extend(["-q", quality])
 
-    if consensus is not None:
+    if consensus is not None and calc_xs:
         gem_2_sam_p.extend(['-s', _prepare_splice_consensus_parameter(consensus)])
 
     if single_end:
@@ -830,7 +859,7 @@ def gem2sam(input, index=None, output=None,
         gem_2_sam_p.append("-c")
 
     # GT-25 transform id's
-    process = utils.run_tool(gem_2_sam_p, input=input, output=output, name="GEM-2-sam", write_map=True, clean_id=True, append_extra=False)
+    process = utils.run_tool(gem_2_sam_p, input=input, output=output, name="GEM-2-sam", write_map=True, clean_id=True, append_extra=False, raw=raw)
     return _prepare_output(process, output=output, quality=quality)
 
 
@@ -899,7 +928,7 @@ def sam2bam(input, output=None, sorted=False, tmpdir=None, mapq=None, threads=1,
 def bamIndex(input, output=None):
     in_name = input
     if not isinstance(input, basestring):
-        in_name = input.file
+        in_name = input.filename
     bam_p = ['samtools', 'index', in_name]
     if output is None:
         output = in_name + ".bai"
@@ -911,7 +940,7 @@ def bamIndex(input, output=None):
     return output
 
 
-def compute_transcriptome(max_read_length, index, junctions, substract=None):
+def compute_transcriptome(max_read_length, index, junctions, substract=None, output_name=None):
     """Compute the transcriptome based on a set of junctions. You can optionally specify
     a *substract* junction set. In that case only junctions not in substract are passed
     to compute the transcriptome.
@@ -923,14 +952,18 @@ def compute_transcriptome(max_read_length, index, junctions, substract=None):
     junctions -- path to the junctions file
     substract -- additional juntions that are not taken into account and substracted from the main junctions
     """
+    if output_name is None:
+        output_name = os.path.basename(junctions)
     transcriptome_p = [
-        executables['compute-transcriptome'],
-        str(max_read_length),
-        index,
-        junctions
+        executables['gem-rna-tools'],
+        'compute-transcriptome',
+        '-l', str(max_read_length),
+        '-I', index,
+        '-j', junctions,
+        '-o', output_name
     ]
     if substract is not None:
-        transcriptome_p.append(substract)
+        transcriptome_p.extend(['-J', substract])
 
     process = utils.run_tools([transcriptome_p], input=None, output=None, name="compute-transcriptome")
     if process.wait() != 0:
@@ -984,19 +1017,84 @@ def index(input, output, content="dna", threads=1):
     return os.path.abspath("%s.gem" % output)
 
 
-def hash(input, output):
-    """Run the gem-retriever on the given input and create a hash
-    version of a genome that can be used by the retriever to query
-    the reference by chromosome and coordinates
+def gtfcounts(inputs, annotation, output=None, json_output=None, threads=1,
+              counts=None, weight=True, multimaps=False, exon_threshold=0,
+              paired=False):
+    """Run the count stats. This returns the gtf count stats as dictionary"""
+    p = [
+        executables['gt.gtfcount'],
+        '--threads', str(threads),
+        '-f', 'both',
+        '-a', annotation
+    ]
+    if paired:
+        p.append("-p")
+
+    if counts is not None:
+        p.extend(["-g", counts])
+        if weight:
+            p.append("-w")
+        if multimaps:
+            p.append("-m")
+        if exon_threshold > 0:
+            p.extend(["-e", str(exon_threshold)])
+
+    from subprocess import PIPE
+    process = utils.run_tools([p], name="gtfcounts", input=inputs,
+                              output=output, raw=True, write_map=True,
+                              logfile=PIPE)
+
+    if json_output is not None:
+        json_output = open(json_output, 'w')
+
+    lines = []
+    for line in process.processes[0].process.stderr:
+        lines.append(line.strip())
+        if json_output is not None:
+            json_output.write(line)
+
+    if json_output is not None:
+        json_output.close()
+
+    if process.wait() != 0:
+        raise ValueError("Error while running gt.gtfcounts")
+    import json
+    return json.loads("\n".join(lines))
+
+
+def stats(inputs, output=None, json_output=None, threads=1, paired=False):
+    """Run the count gt.stats with all options enabled. The function returns the parsed
+    json stats.
     """
     p = [
-        executables['gem-retriever'],
-        'hash', input, output
+        executables['gt.stats'],
+        '-a',
+        '-t', str(threads),
+        '-f', 'both'
     ]
-    process = utils.run_tools([p], name="gem-retriever")
+    if paired:
+        p.append("-p")
+
+    from subprocess import PIPE
+    process = utils.run_tools([p], name="stats", input=inputs,
+                              output=output, raw=True, write_map=True,
+                              force_debug=True, logfile=PIPE)
+    if json_output is not None:
+        json_output = open(json_output, 'w')
+
+    lines = []
+    for line in process.processes[0].process.stderr:
+        lines.append(line.strip())
+        if json_output is not None:
+            json_output.write(line)
+
+    if json_output is not None:
+        json_output.close()
+
     if process.wait() != 0:
-        raise ValueError("Error while executing the gem-retriever")
-    return os.path.abspath(output)
+        raise ValueError("Error while running gt.stats")
+    import json
+    return json.loads("\n".join(lines))
 
 
 def _compressor(threads=1):
@@ -1009,100 +1107,89 @@ def _compressor(threads=1):
         return [pigz, "-p", str(threads), "-"]
 
 
-class merger(object):
-    """Merge all mappings from the source files into the target file.
-    The target file must contain all available mappings and the sort order
-    of all files must be the same.
+def merge(master, slaves, output=None, paired=False, same_content=False,
+          compress=False, threads=1):
+    """Merge the content of the master with the content
+    of the salve(s).
 
-    target -- either an open file descriptor or a file name.
-    source -- an open file descriptor to a single source file, a file name or a
-              list of file names
-    exclusive - if set the True, the next mappings are take
-                exlusively if the reads is not mapped at all. The default
-                is False, where all mappings are merged
-    paired -- merging paried reads
     """
-
-    def __init__(self, target, source, paired=False):
-        if target is None:
-            raise ValueError("No target file specified")
-        if source is None:
-            raise ValueError("No source file specified")
-
-        self.target = target
-        self.source = source
-        self.paired = paired
-        self._output = None
-        self._input = None
-
-    def __iter__(self):
-        files = [self.target]
-        files.extend(self.source)
-        m = gt.merge(files)
-        return m.__iter__()
-
-    def __async_merge(self, threads):
-        of = gt.OutputFile(self._output, clean_id=False, append_extra=True)
-        files = [self.target]
-        files.extend(self.source)
-        gt.merge(files).write_stream(of, write_map=True, threads=threads)
-        of.close()
-        self._output.close()
-
-    def merge_async(self, threads=1, paired=False, same_content=False):
-        (r, w) = os.pipe()
-        self._input = os.fdopen(r, 'r')
-        self._output = os.fdopen(w, 'w')
-        process = mp.Process(target=merger.__async_merge, args=(self, threads))
-        gem.utils.register_process(process)
-
-        def ww():
-            process.join()
-        process.wait = ww
-        process.start()
-        self._output.close()
-        return gt.InputFile(self._input, process=process)
-
-    def merge(self, output, threads=1, paired=False, same_content=False, compress=False):
-        if same_content:
-            if output is None:
-                (r, w) = os.pipe()
-                self._input = os.fdopen(r, 'r')
-                self._output = os.fdopen(w, 'w')
-                of = gt.OutputFile(self._output, clean_id=False, append_extra=True)
-                files = [self.target]
-                files.extend(self.source)
-                process = gt.merge(files).write_stream(of, write_map=True, threads=threads, async=True)
-                self._output.close()
-                return gt.InputFile(self._input, process=process)
-                #return self.merge_async(threads=threads, paired=paired, same_content=same_content)
-
-            p = None
-            out = output
-            if compress and not output.endswith(".gz"):
-                output += ".gz"
-            if compress:
-                p = subprocess.Popen(_compressor(threads=max(1, threads / 2)), stdout=open(output, 'wb'), stdin=subprocess.PIPE, close_fds=True)
-                out = p.stdin
-
-            logging.debug("Using merger with %d threads and same content" % (threads))
-            files = [self.target]
-            files.extend(self.source)
-            merger = gt.merge(files)
-            out = gt.OutputFile(output)
-
-            merger.write_stream(out, write_map=True, threads=threads)
-            return gt.InputFile(output)
+    merge_out = subprocess.PIPE
+    if output is not None:
+        if output == sys.stdout:
+            merge_out = output
+            output = None
         else:
-            logging.debug("Using unpaired merger")
-            files = [self.target]
-            files.extend(self.source)
-            merger = gt.merge(files)
-            of = gt.OutputFile(output, clean_id=False, append_extra=True)
-            for m in merger:
-                of.write(m, write_map=True)
-            of.close()
-        return gt.InputFile(output)
+            if not compress:
+                merge_out = output
+                if isinstance(output, basestring):
+                    merge_out = open(output, 'wb')
+            else:
+                p = subprocess.Popen(_compressor(threads=max(1, threads / 2)),
+                                     stdout=open(output, 'wb'),
+                                     stdin=subprocess.PIPE, close_fds=True)
+                merge_out = p.stdin
+
+    # create tmpdir for the fifos
+    tmpdir = tempfile.mkdtemp()
+    gem.files.delete_on_exit.append(tmpdir)
+    current_master = master
+    current_process = None
+    for i, slave in enumerate(slaves):
+        current_output = None
+        if i == (len(slaves) - 1):
+            # last one
+            current_output = merge_out
+        current_process = _merge_two(current_master, slave, current_output,
+                                     tmpdir, i, paired, same_content, threads)
+        current_master = current_process.stdout
+    return _prepare_output(current_process, output=output)
+
+
+def _merge_two(master, slave, output, tmpdir, count,
+               paired=False, same_content=False, threads=1):
+    """Helper function to merge to files. Return the merging
+    process.
+    """
+    pa = [executables['gt.mapset'], '-C', 'merge-map', '-t', str(threads)]
+    if paired:
+        pa.append('-p')
+    if same_content:
+        pa.append('-s')
+
+
+    inmaster = master
+    if isinstance(inmaster, basestring):  # from string
+        inmaster = gem.files.open_file(inmaster)
+    elif hasattr(inmaster, 'stdout'):  # from process
+        inmaster = inmaster.stdout
+    elif isinstance(inmaster, gt.InputFile):  # from gt input file
+        inmaster = inmaster.raw_stream()
+
+    inslave = slave
+    fifo_in = None
+    fifo_out = None
+    if not isinstance(inslave, basestring):
+        # create a fifo and
+        filename = os.path.join(tmpdir, "%d" % count)
+        os.mkfifo(filename)
+        if not hasattr(inslave, 'stdout'):
+            inslave = inslave.raw_stream()
+        fifo_in = inslave
+        inslave = filename
+
+    if output is None:
+        output = subprocess.PIPE
+    elif isinstance(output, basestring):
+        output = open(output, 'wb')
+
+    pa.append('--i1')
+    pa.append(inslave)
+
+    p = subprocess.Popen(pa, stdin=inmaster, stdout=output)
+    if fifo_in is not None:
+        fifo_out = open(filename, 'wb')
+        subprocess.Popen(['cat'], stdin=fifo_in, stdout=fifo_out)
+    return p
 
 
 def _is_i3_compliant(stream):
