@@ -8,7 +8,7 @@ import sys
 from sys import exit
 import signal
 
-__VERSION__ = "1.6.1"
+__VERSION__ = "1.7"
 
 
 def gemtools():
@@ -21,14 +21,16 @@ def gemtools():
 
         commands = {
             "index": gem.production.Index,
-            "hash": gem.production.Hash,
             "rna-pipeline": gem.production.RnaPipeline,
             "t-index": gem.production.TranscriptIndex,
+            "gtfcount": gem.production.GtfCount,
             "merge": gem.production.Merge,
+            "convert": gem.production.Convert,
             "gtf-junctions": gem.production.Junctions,
             "denovo-junctions": gem.production.JunctionExtraction,
             "stats": gem.production.Stats,
-            "report": gem.production.StatsReport
+            "filter": gem.production.Filter,
+            "report": gem.production.StatsReport,
         }
         instances = {}
 
@@ -42,30 +44,15 @@ def gemtools():
         if args.loglevel is not None:
             gem.loglevel(args.loglevel)
 
-        # register cleanup signal handler
-        def cleanup_in_signal(signal, frame):
-            # cleanup
-            gem.utils.teminate_processes()
-            gem.files._cleanup()
-
-        signal.signal(signal.SIGINT, cleanup_in_signal)
-        signal.signal(signal.SIGQUIT, cleanup_in_signal)
-        signal.signal(signal.SIGHUP, cleanup_in_signal)
-        signal.signal(signal.SIGTERM, cleanup_in_signal)
-
         try:
             instances[args.command].run(args)
         except gem.utils.CommandException, e:
             sys.stderr.write("%s\n" % (str(e)))
             exit(1)
     except KeyboardInterrupt:
-        gem.utils.teminate_processes()
-        gem.files._cleanup()
         exit(1)
     finally:
-        # cleanup
-        gem.utils.teminate_processes()
-        gem.files._cleanup()
+        pass
 
 if __name__ == "__main__":
     gemtools()
