@@ -17,11 +17,12 @@ def extract_denovo_junctions(input, minsplit=4, maxsplit=2500000, sites=None, co
                 to be considered, default is 0 and therefore disabled
     """
     splits2junctions_p = [
-        gem.executables['splits-2-junctions'],
-        str(minsplit),
-        str(maxsplit),
-        str(max_junction_matches),
-        str(threads)
+        gem.executables['gem-rna-tools'],
+        'splits-2-junctions',
+        '--min-split-size', str(minsplit),
+        '--max-split-size', str(maxsplit),
+        '--max-matches', str(max_junction_matches),
+        '--threads', str(threads)
     ]
     p = gem.utils.run_tool(splits2junctions_p, input=input, write_map=True)
 
@@ -129,11 +130,11 @@ class append_xs_filter(object):
     # pattern to identify split maps
     split_re = re.compile(".*N.*")
 
-    def __init__(self, index_hash):
+    def __init__(self, index):
         """Create a new filter providing the .hash file
         of the genome reference
         """
-        self.retriever = gem.utils.retriever(index_hash)
+        self.retriever = gem.utils.retriever(index)
 
     def filter(self, input):
         if input[0] == "@":
